'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

let convertCode = (() => {
	var _ref = _asyncToGenerator(function* (application, file, settings) {
		const options = settings.opts || { globals: {} };
		const source = (file.buffer || '').toString('utf-8');
		const cached = getCached(source, source, application.cache);

		const component = cached.isCached ? null : getComponent(source, {
			application: application, options: options, manifest: file.pattern.manifest, parseKey: cached.parseKey
		});

		const dependencyNames = cached.dependencyNames || (0, _lodash.uniq)((yield (0, _getResolvableDependencies2.default)(component, file)));

		yield Promise.all(dependencyNames.map((() => {
			var _ref2 = _asyncToGenerator(function* (name) {
				const dependencies = file.dependencies;

				const dependency = dependencies[name];
				if (typeof dependency === 'undefined') {
					return Promise.resolve();
				}
				dependencies[name] = yield convertCode(application, dependency, settings);
				return dependencies[name];
			});

			return function (_x4) {
				return _ref2.apply(this, arguments);
			};
		})()));

		const program = cached.isCached ? null : (0, _babylonJsx2.default)(component, 'React.createElement');

		var _ref3 = cached.isCached ? cached : (0, _babelGenerator2.default)(program.program);

		const code = _ref3.code;

		application.cache.set(cached.transformKey, false, { code: code, dependencyNames: dependencyNames });

		file.buffer = `'use strict';\n${ code }`;
		file.meta.react = (0, _lodash.merge)({}, file.meta.react, {
			ast: program,
			dependencyNames: dependencyNames
		});
		return file;
	});

	return function convertCode(_x, _x2, _x3) {
		return _ref.apply(this, arguments);
	};
})();

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _lodash = require('lodash');

var _md = require('md5');

var _md2 = _interopRequireDefault(_md);

var _pascalCase = require('pascal-case');

var _pascalCase2 = _interopRequireDefault(_pascalCase);

var _babylonJsx = require('babylon-jsx');

var _babylonJsx2 = _interopRequireDefault(_babylonJsx);

var _createReactComponent = require('./create-react-component');

var _createReactComponent2 = _interopRequireDefault(_createReactComponent);

var _getResolvableDependencies = require('./get-resolvable-dependencies');

var _getResolvableDependencies2 = _interopRequireDefault(_getResolvableDependencies);

var _parse = require('./parse');

var _parse2 = _interopRequireDefault(_parse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function getReactTransformFunction(application, config) {
	return (() => {
		var _ref4 = _asyncToGenerator(function* (file, _, configuration) {
			const settings = (0, _lodash.merge)({}, config, configuration);
			return yield convertCode(application, file, settings);
		});

		return function (_x5, _x6, _x7) {
			return _ref4.apply(this, arguments);
		};
	})();
}

exports.default = application => {
	const config = application.configuration.transforms.react;

	return getReactTransformFunction(application, config);
};

function getComponent(source, context) {
	const application = context.application;
	const manifest = context.manifest;
	const parseKey = context.parseKey;
	const options = context.options;


	const ast = application.cache.get(parseKey, false) || (0, _parse2.default)(source);

	application.cache.set(parseKey, null, ast);

	// manifest.name is used as name for wrapped components
	const manifestName = manifest.name;

	// Get the component ast
	return (0, _createReactComponent2.default)(ast, (0, _pascalCase2.default)(manifestName), options.globals);
}

function getCached(source, options, cache) {
	const hash = (0, _md2.default)(source);

	const parseKey = ['react', 'parse', hash, JSON.stringify(options.globals)].join(':');

	const transformKey = ['react', 'transform', hash, JSON.stringify(options.globals)].join(':');

	const cached = cache.get(transformKey, false) || {
		dependencyNames: null, code: null
	};

	const isCached = cached.code !== null;

	return {
		code: cached.code,
		dependencyNames: cached.dependencyNames,
		isCached: isCached,
		parseKey: parseKey,
		transformKey: transformKey
	};
}
module.exports = exports['default'];